/***********************************************************************************************
	File Name:		CommitMailConsole.cs
	Purpose:		Console application for emailing new repository commits to team members
	Author:			Jon Gilkison <jon.gilkison@interfacelab.com>
	Copyright:		2004 Interfacelab LLC                                                             
	Date:			3/23/2004
 ***********************************************************************************************/

using System;
using System.Collections.Specialized;
using System.IO;
using System.Diagnostics;
using System.Text;
using System.Text.RegularExpressions;

namespace commitmailer
{
	/// <summary>
	/// Main class for console application.
	/// </summary>
	class CommitMailConsole
	{
		/// <summary>
		/// String holding the repository passed in as an argument to the application.
		/// </summary>
		static string strRepos=string.Empty;

		/// <summary>
		/// String holding the new revision number, passed in as an argument to the application.
		/// </summary>
		static string strRev=string.Empty;

		/// <summary>
		/// Path to the svnlook.exe application, specified in the application's config section.
		/// </summary>
		static string strSvnLook=string.Empty;

		/// <summary>
		/// Regular expression for parsing the info returned from svnlook's changed command.
		/// </summary>
		static Regex rxUpdateParse=new Regex(@"(?<op>[a-zA-Z])(?:[ ]*)(?<file>[^\r\n]+)",RegexOptions.Compiled | RegexOptions.IgnoreCase | RegexOptions.Multiline);

		/// <summary>
		/// Invokes svnlook.exe and returns it's output as a string.
		/// </summary>
		/// <param name="strOp">The svnlook operation to invoke (info, dir-changed, changed, etc.)</param>
		/// <returns>The output of the svnlook.exe application</returns>
		static string  ReadSVNLook(string strOp)
		{
			Process procLook = new Process();
			ProcessStartInfo psiLook = new ProcessStartInfo(strSvnLook);
			psiLook.Arguments=string.Format("{0} {1} -r {2}",strOp, strRepos,strRev);
			psiLook.UseShellExecute = false;
			psiLook.RedirectStandardOutput = true;
			procLook.StartInfo = psiLook;
			procLook.Start();
			StreamReader srProc = procLook.StandardOutput;
			string strResult= srProc.ReadToEnd();
			procLook.WaitForExit();

			return strResult;
		}

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			// Make sure the correct arguments are passed in.
			if (args.Length!=2)
			{
				Console.WriteLine("Command arguments missing.");
				Console.WriteLine("");
				Console.WriteLine("Usage:");
				Console.WriteLine("\tCommitMailer.exe [repository] [revision_number]");
				return;
			}

			strRepos=args[0];
			strRev=args[1];
			strSvnLook=System.Configuration.ConfigurationSettings.AppSettings["SVNLook"];

			// make sure SVNLook exists ...
			if (!File.Exists(strSvnLook))
			{
				Console.WriteLine("SVNLook could not be found.");
				Console.WriteLine("");
				Console.WriteLine("Please make sure the appropriate directory is in the <appsettings> section of the application's config file.");
				return;
			}

			// Invoke SVN Look and get the information about the commit
			string strCommitInfo = ReadSVNLook("info");

			// break the output into individual lines.
			string[] arrInfo=strCommitInfo.Split('\n');

			// make sure we have what we need ...
			if (arrInfo.Length<2)
			{
				Console.WriteLine("Information is from SVNLook is invalid or missing.");
				Console.WriteLine("");
				Console.WriteLine("Output:");
				Console.WriteLine(strCommitInfo);
				return;
			}

			// parse out the user and the date of the commit
			string strUser=arrInfo[0];
			string[] arrDateTime=arrInfo[1].Split(' ');
			DateTime dtCommitTime=DateTime.Parse(string.Format("{0} {1}",arrDateTime[0],arrDateTime[1]));

			// find out which files changed.
			string strChanged = ReadSVNLook("changed");


			// Collections to hold our added, deleted and modified files.
			StringCollection scAdded=new StringCollection();
			StringCollection scDeleted=new StringCollection();
			StringCollection scModified=new StringCollection();

			// Run the regex and iterate through the matches, adding them to the appropriate collection.
			MatchCollection mcUpdates=rxUpdateParse.Matches(strChanged);
			foreach(Match m in mcUpdates)
				if (m.Groups.Count>0)
					switch(m.Groups["op"].Value)
					{
						case "A":
							scAdded.Add(m.Groups["file"].Value);
							break;
						case "D":
							scDeleted.Add(m.Groups["file"].Value);
							break;
						default:
							scModified.Add(m.Groups["file"].Value);
							break;
					}

			// Build the body of the email.
			StringBuilder sbBody=new StringBuilder();
			sbBody.Append(string.Format("Author: {0}\r\n",strUser));
			sbBody.Append(string.Format("Date: {0}\r\n",dtCommitTime));
			sbBody.Append(string.Format("New Revision: {0}\r\n",strRev));

			// If any files were added, add this to the email body.
			if (scAdded.Count>0)
			{
				sbBody.Append("\r\n");
				sbBody.Append("Added:\r\n");
				foreach(string st in scAdded)
					sbBody.Append(string.Format("\t{0}\r\n",st));
			}

			// If any files were deleted, add this to the email body.
			if (scDeleted.Count>0)
			{
				sbBody.Append("\r\n");
				sbBody.Append("Deleted:\r\n");
				foreach(string st in scDeleted)
					sbBody.Append(string.Format("\t{0}\r\n",st));
			}

			// If any files were modified, add this to the email body.
			if (scModified.Count>0)
			{
				sbBody.Append("\r\n");
				sbBody.Append("Modified:\r\n");
				foreach(string st in scModified)
					sbBody.Append(string.Format("\t{0}\r\n",st));
			}

			// fetch the configuration settings for whom to send the mail to, and who the mail is from.
			string strMailFrom=System.Configuration.ConfigurationSettings.AppSettings["MailFrom"];
			string strMailTo=System.Configuration.ConfigurationSettings.AppSettings["MailTo"];

			// Send the email.
			SimpleMailer.Send(System.Configuration.ConfigurationSettings.AppSettings["SMTPServer"],strMailFrom,strMailTo,"","",string.Format("New revision {0} for {1}",strRev,strRepos),sbBody.ToString());

			// done.
		}
	}
}
